/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     navgoaldata.cc
 * @author   Roland Philippsen
 * @date     14.09.2006
 * @brief    Converting and extracting navgoal data to and from xml-structure
 */

#include <sstream>
#include <math.h>

#include "NavigationGoalData.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

std::string NavigationGoalData::getBaseTag() {
	return "NAVIGATIONGOAL";
}

std::string NavigationGoalData::getTemplate() {

	return "<" + getBaseTag() + ">"
		+ "<GENERATOR/>"
		+ airobots::Timestamp::getTemplate()
		+ "<POSITION ref=\"world\" kind=\"absolute\" x=\"\" y=\"\" theta=\"\" />"
		+ "<TOLERANCES><DR quant=\"\"/><DTHETA quant=\"\"/></TOLERANCES>"
		+ "<TYPE/>"
		+ "</" + getBaseTag() + ">";

}

NavigationGoalData::NavigationGoalData(){

}

NavigationGoalData::NavigationGoalData(std::string generator, float x, float y,
		float theta, float dr, float dtheta,
		NavigationGoalData::NavgoalType type) :
	PositionData(x, y, theta), dR(dr), dTheta(dtheta), type(
			type) {
}

float NavigationGoalData::getDR() const {
	return dR;
}

float NavigationGoalData::getDTheta() const {
	return dTheta;
}

NavigationGoalData::NavgoalType NavigationGoalData::getType() const {
	return type;
}

void NavigationGoalData::setDR(float dR) {
	this->dR = dR;
}

void NavigationGoalData::setDTheta(float dTheta) {
	this->dTheta = dTheta;
}

void NavigationGoalData::setType(NavgoalType type) {
	this->type = type;
}

bool NavigationGoalData::operator==(const NavigationGoalData &other) const {

	return (other.getGenerator() == getGenerator()) && (other.getTimestamp()
			== getTimestamp()) && (other.getX() == getX()) && (other.getY()
			== getY()) && (other.getTheta() == getTheta()) && (other.dR == dR)
			&& (other.dTheta == dTheta) && (other.type == type);

}

std::ostream& airobots::operator<<(std::ostream& o,
		NavigationGoalData const& ngd) {
	o << "NavigationGoalData(generator = " << ngd.getGenerator()
			<< ", timestamp = " << ngd.getTimestamp() << ", x = " << ngd.getX()
			<< ", y = " << ngd.getY() << ", theta = " << ngd.getTheta()
			<< ", dR = " << ngd.getDR() << ", dTheta = " << ngd.getDTheta()
			<< ", type = " << ngd.getType() << ")";
	return o;
}

void NavigationGoalData::sanitizeLocation(xmltio::Location &loc) const {
	
}

void NavigationGoalData::fillInto(xmltio::Location &loc) const {

	PositionData::fillInto(loc);

	loc[XPath("TOLERANCES/DR/@quant")] = getDR();
	loc[XPath("TOLERANCES/DTHETA/@quant")] = min(getDTheta(), (float) M_PI);

	std::string type;
	switch (getType()) {
	case NavigationGoalData::GLOBAL:
		type = "GLOBAL";
		break;
	case NavigationGoalData::LOCAL:
		type = "LOCAL";
		break;
	default:
		type = "LOCAL";
	}
	loc[XPath("TYPE")] = type;

}

void NavigationGoalData::fromLocation(const xmltio::Location &loc_, NavigationGoalData &ngd) {

	PositionData::fromLocation(loc_, ngd);

	ngd.setDR(extract<float> (loc_[XPath("TOLERANCES/DR/@quant")]));
	ngd.setDTheta(min(
			extract<float> (loc_[XPath("TOLERANCES/DTHETA/@quant")]).get(),
			(float) M_PI));

	LocationPtr typeLoc = find(loc_, XPath("TYPE"));
	NavigationGoalData::NavgoalType type = NavigationGoalData::LOCAL;
	if (typeLoc) {
		if (typeLoc->getTextValue() == "GLOBAL") {
			type = NavigationGoalData::GLOBAL;
		}
	}
	ngd.setType(type);

}

