#ifndef REGIONDATA_H_
#define REGIONDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "Type.h"

namespace airobots {

/// triple holds three objects of arbitrary type.
template<class _T1, class _T2, class _T3>
struct triple {
	typedef _T1 first_type; ///<  @c first_type is the first bound type
	typedef _T2 second_type; ///<  @c second_type is the second bound type
	typedef _T2 third_type; ///<  @c third_type is the third bound type

	_T1 first; ///< @c first is a copy of the first object
	_T2 second; ///< @c second is a copy of the second object
	_T3 third; ///< @c third is a copy of the third object

	triple() :
		first(), second(), third() {
	}

	/** Two objects may be passed to a @c pair constructor to be copied.  */
	triple(const _T1& __a, const _T2& __b, const _T3& __c) :
		first(__a), second(__b), third(__c) {
	}

	/** There is also a templated copy ctor for the @c pair class itself.  */
	template<class _U1, class _U2, class _U3>
	triple(const triple<_U1, _U2, _U3>& __p) :
		first(__p.first), second(__p.second), third(__p.third) {
	}
};

/// Two pairs of the same type are equal iff their members are equal.
template<class _T1, class _T2, class _T3>
inline bool operator==(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return __x.first == __y.first && __x.second == __y.second && __x.third == __y.third;
}

/// <http://gcc.gnu.org/onlinedocs/libstdc++/20_util/howto.html#pairlt>
template<class _T1, class _T2, class _T3>
inline bool operator<(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return __x.first < __y.first || (!(__y.first < __x.first) && __x.second < __y.second)
			|| (!(__y.first < __x.first) && !(__y.second < __x.second) && __x.third < __y.third);
}

/// Uses @c operator== to find the result.
template<class _T1, class _T2, class _T3>
inline bool operator!=(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return !(__x == __y);
}

/// Uses @c operator< to find the result.
template<class _T1, class _T2, class _T3>
inline bool operator>(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return __y < __x;
}

/// Uses @c operator< to find the result.
template<class _T1, class _T2, class _T3>
inline bool operator<=(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return !(__y < __x);
}

/// Uses @c operator< to find the result.
template<class _T1, class _T2, class _T3>
inline bool operator>=(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return !(__x < __y);
}

/**
 * Defines a plane in 3d.
 *
 * @author lziegler
 */
class Path3DData: public Type {

public:
	enum Unit {
		MILLIMETER, CENTIMETER, METER
	};

	enum Scope {
		LOCAL, GLOBAL
	};
	Scope scope;

	std::vector<triple<float, float, float> > polygon;
	std::vector<triple<float, float, float> > scalePolygon(
			std::vector<triple<float, float, float> > poly, double factor) const;

public:

	static std::string getBaseTag();
	static std::string getTemplate();

	std::vector<triple<float, float, float> > getPolygon(Unit unit) const;
	void setPolygon(const std::vector<triple<float, float, float> > &polygon, Unit unit);

	void setScope(const Scope &scope);
	Scope getScope() const;

	virtual bool operator==(const Path3DData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, Path3DData &type);

	Path3DData();
	Path3DData(std::string generator);
};

}

#endif /* REGIONDATA_H_ */
