 /** ==========================================================================
 * 
 * @file		SoundPercept.cpp --
 * @brief		Source file for class representing a single sound percept.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * 
 * ========================================================================== 
 **/

#include "SoundPercept.h"

using namespace airobots;

// ==========================================================================
// Public stuff.
// ==========================================================================

SoundPercept::SoundPercept(int id) : id(id) {
	label = "unknown";
	spectraCounter = 0;
}

SoundPercept::~SoundPercept() {

}

std::string
SoundPercept::getBaseTag() {
	return "SOUND";
}

std::string
SoundPercept::getTemplate() {

	// Add single information values.
	std::stringstream stream;
	stream << "<" << getBaseTag() << ">";
	stream << 	Timestamp::getTemplate();
	stream << 	"<GENERATOR />";
	stream << 	"<LABEL value=\"\" />";
	stream <<		"<TRACKING id=\"\" />";
	stream <<		"<MAINSPECBANDS nrValues=\"0\" />";
	stream <<		"<LOCEST nrValues=\"0\" />";
	stream << "</SOUND>";

	return stream.str();

}

std::string
SoundPercept::getPerceptName() {
	return "sound";
}

std::string
SoundPercept::toXml() {	

	unsigned int i = 0;

	// Add single information values.
	std::stringstream stream;
	stream << "<SOUND>";
	stream << 	"<LABEL value=\"" << label << "\" />";
	stream <<		"<TRACKING id=\"" << id << "\" />";

	// Add current estimated main spectrum bands.
	unsigned int nrAverages = averages.size();
	stream <<		"<MAINSPECBANDS nrValues=\"" << nrAverages << "\" ";	
	for (i = 0; i < nrAverages; i++) {
		stream << "value" << i << "=\"" << (int) (averages[i] + 0.5) << "\" ";
	}
	stream << 	"/>";

	// Add current sound source localisation estimation.
	unsigned int nrAngles = angles.size();
	stream <<		"<LOCEST nrValues=\"" << nrAngles << "\" ";
	for (i = 0; i < nrAngles; i++) {
		stream << "value" << i << "=\"" << angles[i] << "\" ";	
	}
	stream <<		"/>";

	stream << "</SOUND>";

	return stream.str();
}

void
SoundPercept::addSpectrum(std::vector<int> values) {
	spectraCounter++;
	if (spectraCounter == 1) {
		for (unsigned int i = 0; i < values.size(); i++) {
			averages.push_back(values[i]);
		}
		return;
	}
	for (unsigned int i = 0; i < values.size(); i++) {
		float oldValue = averages[i];
		float newValue = values[i];
		float newMean = incrementalMean(oldValue, newValue, spectraCounter);
		averages[i] = newMean;
	}
}

void 
SoundPercept::getCurrentMaxBands(std::vector<int> &values) {
	for (unsigned int i = 0; i < averages.size(); i++) {
		values.push_back(averages[i] + 0.5);
	}
}


void
SoundPercept::setLabel(std::string label) {
	this->label = label;
}

std::string
SoundPercept::getLabel() {
	return label;
}

void
SoundPercept::setLocationEstimation(std::vector<float> angles) {
	this->angles = angles;
}

void
SoundPercept::writeToStream(std::fstream *outputstream) {
	(*outputstream) << "Sound percept: " << id << std::endl;
	for (unsigned int i = 0; i < averages.size(); i++) {
		(*outputstream) << averages[i] << " ";
	}
	(*outputstream) << std::endl;
}

void
SoundPercept::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[xmltio::XPath("LABEL/@value")] = label;
	loc[xmltio::XPath("TRACKING/@id")] = id;

	// Add main spectrum bands to document.
	unsigned int nrValues = averages.size();
	loc[xmltio::XPath("MAINSPECBANDS/@nrValues")] = nrValues;

	xmltio::Location mainspec(loc, "MAINSPECBANDS");
	std::stringstream tmp;
	unsigned int i = 0;
	for (i = 0; i < nrValues; i++) {
		tmp.str(""); tmp.clear();
		tmp << "value" << i;
		mainspec[tmp.str().c_str()] = (int) (averages[i] + 0.5);
	}
	
	// Add location estimations.
	xmltio::Location locest(loc, "LOCEST");
	nrValues = angles.size();
	loc[xmltio::XPath("LOCEST/@nrValues")] = nrValues;
	for (i = 0; i < nrValues; i++) {
		tmp.str(""); tmp.clear();
		tmp << "value" << i;
		locest[tmp.str().c_str()] = angles[i];
	}

}

void
SoundPercept::sanitizeLocation(xmltio::Location &loc) const {

}

void
SoundPercept::fromLocation(const xmltio::Location &loc, SoundPercept &percept) {
	std::cout << "!!! SoundPercept::fromLocation is not implemented yet !!!" << std::endl;
}

// ==========================================================================
// Private stuff.
// ==========================================================================

float 
SoundPercept::incrementalMean(float oldValue, float newValue, float addedValues) {
	float value = ((addedValues - 1) / addedValues) * oldValue + (1 / addedValues) * newValue;
	return value;
}

