#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/GlobalPlan.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(GlobalPlanTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		GlobalPlan original("test");
		original.setTimestamp(Timestamp(123, 456));
		original.addWaypoint(NavigationGoalData("testing", 10, 12.5, 0.34, 0.4, 0.5));
		original.addWaypoint(NavigationGoalData("bla", 100, 102.5, 0.304, 0.04, 0.24));

		Location originalLoc(GlobalPlan::getTemplate(), GlobalPlan::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true);

		GlobalPlan parsed = extract<GlobalPlan>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.size(), original.size());
		list<NavigationGoalData>::const_iterator parsedIt = parsed.begin();
		list<NavigationGoalData>::const_iterator originalIt = original.begin();
		while (parsedIt != parsed.end()) {
			BOOST_REQUIRE_EQUAL(*parsedIt, *originalIt);

			parsedIt++;
			originalIt++;
		}

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("GlobalPlan.xml"), GlobalPlan::getBaseTag());
	GlobalPlan parsed = extract<GlobalPlan>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(3, parsed.size());

}

BOOST_AUTO_TEST_CASE(testLocationReuse) {

	const std::string attributeName = "testattribute";
	const std::string elementName = "ADDITIONAL";

	GlobalPlan original("test");
	original.setTimestamp(Timestamp(123, 456));
	original.addWaypoint(NavigationGoalData("testing", 10, 12.5, 0.34, 0.4, 0.5));
	original.addWaypoint(NavigationGoalData("bla", 100, 102.5, 0.304, 0.04, 0.24));

	Location originalLoc(GlobalPlan::getTemplate(), GlobalPlan::getBaseTag());
	originalLoc = original;

	cout << "adding additional attribute and elements" << endl;
	originalLoc[attributeName.c_str()] = 23;
	Location additionalElement("<" + elementName + "/>", elementName);
	originalLoc.add(additionalElement);
	cout << "changed document" << endl;

	cout << originalLoc.getDocumentText(true);

	// do some parsing and re-serialization into the old location
	GlobalPlan parsedFirstTime = extract<GlobalPlan>(originalLoc);
	originalLoc = parsedFirstTime;
	GlobalPlan parsedSecondTime = extract<GlobalPlan>(originalLoc);

	cout << originalLoc.getDocumentText(true);

	// ensure that the parsed data is still the same and additional elements in
	// the xml still exist
	BOOST_REQUIRE_EQUAL(original.size(), parsedSecondTime.size());
	originalLoc[XPath("@" + attributeName)];
	originalLoc[XPath(elementName)];

}

BOOST_AUTO_TEST_SUITE_END()
