package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;

import java.util.Map;
import java.util.HashMap;

import java.util.List;
import java.util.ArrayList;

import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.Element;
import nu.xom.Attribute;

// ================================== TODO ==================================
// Check if the assigned percepts contains the same information is really 
// necessary?
//
// The trajectory has to be removed from the anchor. 
//
// A composite anchor should be a subclass of an anchor.
// ==========================================================================

/**
 * A general data structur for an anchor. It contains
 * the current percept used to track the object and
 * a automatically generated trajectory.
 *
 * @author dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class Anchor extends Type {

	/** A hash map containing the attributes of the anchor. E.g. size. */
	private HashMap<String, AnchorAttribute> attributes;	

	/** The last percept added to this anchor. */
	private Element percept;

	/** The root element for the document containing the xcf id. */
	private Element rootElement;

	/** The trajectory belonging to this anchor. */
	private Trajectory trajectory;

	/** The type of the anchor. E.g. legs. */
	private String type;

	public static String getBaseTag() {
		return "ANCHOR";
	}

	public Anchor() {				

		this.trajectory = new Trajectory();
		this.attributes = new HashMap<String, AnchorAttribute>();
		this.percept = null;	
		this.type = "unknown";
		
	}

	/** Sets the type of the anchor. E.g.: Legs. */
	public void setType(String type) {
		this.type = type;
	}

	/** Returns the type of the anchor. */
	public String getType() {
		return type;
	}

	/** 
	 * Adds a new attribute to this anchor using the attribute's name
	 * as a key to store the attribute in a hash map. If an attribute
	 * with the same name already exists, it is replaced by the new
	 * attribute.
	 *
	 * @param attr				The attribute to store in the anchor.	
	 */
	public void addAttribute(AnchorAttribute attr) {
		attributes.put(attr.getName(), attr);
	}

	/**
	 * Returns the attribute with the specific name or null if no
	 * attribute with the name exists.
	 *
	 * @param name		The name of the attribute.
	 */
	public AnchorAttribute getAttribute(String name) {

		if (!attributes.containsKey(name))
			return null;

		return attributes.get(name);

	}

	/** Returns all attributes of this anchor. */
	public Map<String, AnchorAttribute> getAttributes() {
		return attributes;
	}

	/** Sets a new trajectory to this anchor. */
	public void setTrajectory(Trajectory trajectory) {
		this.trajectory = trajectory;
	}

	/** Returns the trajectory belonging to this anchor. */
	public Trajectory getTrajectory() {
		return trajectory;
	}

	/** 
	 * Assigns a percept to this anchor. 
	 * 
	 * @param percept		A xml representation of the last percept assigned to the anchor.
	 */
	public void assignPercept(Element percept) {		
		this.percept = (Element) percept.copy();
	}

	/** Returns the percept assigned to this anchor. */
	public Element getPercept() {
		return percept;
	}	

	/** 
	 * Assigns a root element with a xcf id to this anchor.
	 * 
	 * @param element		Root element with unique xcf id.
	 */
	public void assignRootElement(Element element) {
		this.rootElement = (Element) element.copy();
	}

	/** 
	 * Removes the root element from this anchor. 
	 */
	public void removeRootElement() {
		this.rootElement = null;
	}

	/** 
	 * Fills the information in the assigned root element.
	 */
	public Element fillInto() {
		Element copy = (Element) rootElement.copy();
		fillInto(copy);
		return copy;
	}

	@Override
	public boolean equals(Object obj) {

		if (!(obj instanceof Anchor)) 
			return false;

		Anchor anc = (Anchor) obj;
		
		// Check if anchors have the same type.
		boolean sameType = getType().equals(anc.getType());
		if (!sameType)
			return false;

		// Check different attributes of the anchors.
		for (String key : attributes.keySet()) {

			if (anc.getAttribute(key) == null)
				return false;

			AnchorAttribute attr1 = attributes.get(key);
			AnchorAttribute attr2 = anc.getAttribute(key);

			if (!(attr1.equals(attr2)))
				return false;

		}

		// Check if anchors have the same assigned percept. 
		Element percept = anc.getPercept();
		boolean samePercept = false;

		if (percept != null && getPercept() != null) {
			samePercept = percept.toXML().equals(getPercept().toXML());
		}
		
		if (percept == null && getPercept() == null) {
			samePercept = true;
		}			
		if (!samePercept)
			return false;

		return true;

	}

	/** @{inheritDoc} */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);		

		// Add type attribute to parent if it does not exists.
		if (parent.getAttribute("type") == null) {
			parent.addAttribute(new Attribute("type", type));
		} else {
			parent.getAttribute("type").setValue(type);
		}

		// Append different attributes.
		for (AnchorAttribute attr : attributes.values()) {

			Element attrElement = new Element(AnchorAttribute.getBaseTag());
			attr.fillInto(attrElement);
		
			parent.appendChild(attrElement);

		}

		// Add trajectory to tracked element.
		if (trajectory != null) {
			Element trajectoryElement = new Element(Trajectory.getBaseTag());
			trajectory.fillInto(trajectoryElement);
		
			parent.appendChild(trajectoryElement);
		}		

		// Add assigned percept to anchor.
		if (percept != null) {
			Element perceptElement = new Element("PERCEPT");
		
			Element copy = (Element) percept.copy();

			perceptElement.appendChild(copy);
			parent.appendChild(perceptElement);
		}		

	}
	
	public static void fromElement(Element element, Anchor anchor) {

		Type.fromElement(element, anchor);

		// Parse anchor type from root element.
		anchor.setType(ElementParser.getAttributeValue(element, "type"));

		Nodes nodes = element.query("*");
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element elem = (Element) node;
		
				if (elem.getLocalName().equals(AnchorAttribute.getBaseTag())) {

					AnchorAttribute attr = new AnchorAttribute();
					AnchorAttribute.fromElement(elem, attr);

					anchor.addAttribute(attr);

				}	

				if (elem.getLocalName().equals("PERCEPT")) {
					anchor.assignPercept((Element) elem.copy());
				}
		
				if (elem.getLocalName().equals(Trajectory.getBaseTag())) {

					Trajectory parsed = new Trajectory();
					Trajectory.fromElement(elem, parsed);
				
					anchor.setTrajectory(parsed);

				}				

			}

		}
		
	}

}

