package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;
import nu.xom.Attribute;
import nu.xom.Element;

/**
 * Representation of camera position.
 * 
 * @author lziegler
 */
public class CameraPositionData extends Type {

	private double pan = 0;
	private double tilt = 0;
	private double zoom = 0;
	
	/**
	 * Constructor.
	 */
	public CameraPositionData() {
		super();
	}

	/**
	 * Constructor.
	 */
	public CameraPositionData(double pan, double tilt, double zoom) {
		super();
		this.pan = pan;
		this.tilt = tilt;
		this.zoom = zoom;
	}

	/**
	 * Returns the horizontal angle value as radiant for the new position.
	 * 
	 * @return pan [rad]
	 */
	public double getPan() {
		return pan;
	}

	/**
	 * Sets the horizontal angle value as radiant for the new position.
	 * 
	 * @param pan
	 *            Horizontal angle as radiant (0=forward, angles CCW)
	 */
	public void setPan(double pan) {
		this.pan = pan;
	}

	/**
	 * Returns the vertical angle value as radiant for the new position.
	 * 
	 * @return tilt [rad]
	 */
	public double getTilt() {
		return tilt;
	}

	/**
	 * Sets the vertical angle value as radiant for the new position.
	 * 
	 * @param tilt
	 *            Vertical angle as radiant (0=forward, negative = down)
	 */
	public void setTilt(double tilt) {
		this.tilt = tilt;
	}

	/**
	 * Returns the zoom level of the camera as percent of the total zoom.
	 * 
	 * @return zoom [0..1]
	 */
	public double getZoom() {
		assert (zoom >= 0 && zoom <= 1) : "Zoom must be a percent value [0..1].";
		return zoom;
	}

	/**
	 * Sets the zoom as percent of the total zoom of the camera.
	 * 
	 * @param zoom
	 *            desired zoom [0..1]
	 * @throws IllegalArgumentException
	 *             illegal zoom value
	 */
	public void setZoom(double zoom) throws IllegalArgumentException {
		if (zoom < 0 || zoom > 1) {
			throw new IllegalArgumentException(
					"Zoom must be a value in [0..1] but got " + zoom + ".");
		}
		this.zoom = zoom;
	}

	public static String getBaseTag() {
		return "CAMERAPOSITION";
	}

	public static void fromElement(Element element, CameraPositionData type) {

		Type.fromElement(element, type);

		Element panElement = element.getFirstChildElement("PAN");
		type.setPan(ElementParser.getDoubleAttributeValue(panElement, "rad"));

		Element tiltElement = element.getFirstChildElement("TILT");
		type.setTilt(ElementParser.getDoubleAttributeValue(tiltElement, "rad"));

		Element zoomElement = element.getFirstChildElement("ZOOM");
		
		type
				.setZoom(ElementParser.getDoubleAttributeValue(zoomElement,
						"qual"));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element cylindricElement = new Element("PAN");
		parent.appendChild(cylindricElement);
		cylindricElement.addAttribute(new Attribute("rad", String
				.valueOf(getPan())));

		Element azimuthElement = new Element("TILT");
		parent.appendChild(azimuthElement);
		azimuthElement.addAttribute(new Attribute("rad", String
				.valueOf(getTilt())));

		Element zoomElement = new Element("ZOOM");
		parent.appendChild(zoomElement);
		zoomElement.addAttribute(new Attribute("qual", String
				.valueOf(getZoom())));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "CameraPositionData [pan = " + pan + ", tilt = " + tilt
				+ ", zoom = " + zoom + "]";
	}

}
