package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;

import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.Element;
import nu.xom.Attribute;

import java.util.Collection;

import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;

import java.util.Map;
import java.util.HashMap;

// =================================== TODO =================================
// There is a method that adds an anchor only if no anchor with the same type
// exists in this container. Is it logical to have this function here?
//
// Should the new values be computed every time a new anchor is added or an old
// one is updated? Or is it more efficient if they are computed when the information
// of the composite anchor should be converted into a xml document. For now
// they are computed every time an anchor is added to the composite anchor. The
// computation of the fusion is also improvable.
//
// A comparison between composite anchors should not include the containe
// anchors. Otherwise a comparison between composite anchors created from
// different applications could lead to wrong results.
// ==========================================================================

/** 
 * Composite anchors are containers for one or more single anchors that
 * describe the same object in the environment. 
 *
 * @author Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 */
public class CompositeAnchor extends Type {

	/**The attributes of the object described by this composite anchor. */
	private Map<String, AnchorAttribute> attributes;

	/** 
	 * A list with all associated xcf id's from anchors that are used
	 * to compute the information from this composite anchor.
	 */
	private LinkedList<String> xcfIds;

	/**
	 * A list of all perceptual fields where the object described by
	 * this component anchor is anchored in.
	 */
	private LinkedList<String> anchoredFields;

	/** A list of all ratings that are used to build the composite anchor. */
	private HashMap<String, Double> ratings;

	/**
	 * The symbol associated with the composite anchor. E.g.: Person.
	 */
	private String symbol;

	/** Constructor. */
	public CompositeAnchor() {		
		this.attributes = new HashMap<String, AnchorAttribute>();
		this.ratings = new HashMap<String, Double>();
		this.xcfIds = new LinkedList<String>();
		this.anchoredFields = new LinkedList<String>();
		this.symbol = "unknown";
	}

	/** Sets a new symbol to this composite anchor. */
	public void setSymbol(String symbol) {
		this.symbol = symbol;
	}

	/** Returns the symbol associated with the object. */
	public String getSymbol() {	
		return symbol;
	}

	/**
	 * Adds a new attribute to this composite anchor or 
	 * replaces an old attribute that has the same name.
	 */
	public void addAttribute(AnchorAttribute attr) {
		attributes.put(attr.getName(), attr);	
	}	

	/** 
	 * Returns the attribute with the given name or null
	 * if it is not contained in the composite anchor.
	 *
	 * @param name			The name of the attribute.
	 */
	public AnchorAttribute getAttribute(String name) {
		return attributes.get(name);
	}

	/** 
	 * Adds the xcf id of an anchor, and the rating that 
	 * was responsible for the anchor assignment to the
	 * composite anchor, to a list of stored xcf id - rating
	 * correspondences. 
	 *
	 * @param id 				The xcf id of the anchor.
	 * @param rating		The rating that assigns the anchor to
	 *									the composite anchor.
	 */
	public void addAssignment(String id, double rating) {
		System.out.println("Composite anchor adding assignment");
		ratings.put(id, rating);
	}

	/**
	 * Returns the rating for the anchor with xcf id xcfId or
	 * -1 of the anchor is not contained in the composite anchor.
	 * 
	 * @param xcfId				The xcfId of the anchor document.
	 */
	public double getRating(String xcfId) {
		if (ratings.containsKey(xcfId))
			return ratings.get(xcfId);

		return -1;
	}

	/**
	 * Returns the list of all attributes contained in this
	 * composite anchor.
	 */
	public Map<String, AnchorAttribute> getAttributes() {
		return attributes;
	}	

	/**
	 * Adds a new xcf id to the list of ids stored in this
	 * composite anchor. Every id is only stored once.
	 * 
	 * @param id 			The xcf id to add.
	 */
	@Deprecated
	public void addXcfId(String id) {

		if (xcfIds.contains(id))
			return;

		xcfIds.add(id);
		
	}

	/**
	 * Tells if an anchor with id xcfId is used to compute
	 * the values of this anchor.
	 */
	public boolean containsAnchorWithId(String id) {
		return xcfIds.contains(id);
	}

	/**
	 * Returns a list of all ids of anchors that are used
	 * to compute the values of this composite anchor.
	 */
	@Deprecated
	public List<String> getXcfIds() {	
		return xcfIds;
	}

	/**
	 * Removes the xcf id from this composite anchor. 
	 *
	 * @param id			The xcfid to be removed.
	 */
	@Deprecated
	public void removeXcfId(String id) {
		xcfIds.remove(id);
	}

	/** 
	 * Adds a new field where the object is anchored to
	 * the list of all fields in this composite anchor.
	 * Every field is only stored once.
	 * 
	 * @param field				The type of field where the object is anchored.
	 *										Could be an abstract field like: legs.	
	 */
	public void addField(String field) {
		
		if (anchoredFields.contains(field))
			return;
	
		anchoredFields.add(field);

	}	

	/** 
	 * Tells if the object is anchored in the field.
	 * 
	 * @param field				The field where the object might be anchored.
	 */
	public boolean isAnchoredInField(String field) {
		return anchoredFields.contains(field);	
	} 

	/**
	 * Returns a list with all fields the composite anchor is currently
	 * anchored in.
	 */
	public List<String> getAnchoredFields() {
		return anchoredFields;
	}

	/**
	 * Removes the field from this composite anchor.
	 *
	 * @param field			The field name to be removed.
	 */
	public void removeField(String field) {
		anchoredFields.remove(field);
	}

	/**
	 * Removes all stored xcfIds and fields from the anchor.
	 */
	public void clear() {
		xcfIds.clear();
		anchoredFields.clear();
		ratings.clear();
	}

	/**
	 * Tells the number of assigned anchors.
	 */
	public int getNrAnchors() {
		return ratings.size();
	}

	public static String getBaseTag() {
		return "COMPOSITEANCHOR";
	}

	/** @{inheritDoc} */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);
		
		Attribute a = parent.getAttribute("symbol");
		if (a == null) {
			parent.addAttribute(new Attribute("symbol", symbol));
		} else {			
			parent.getAttribute("symbol").setValue(symbol);
		}

		// Append attributes of composite anchor.
		for (AnchorAttribute attr : attributes.values()) {

			Element attrElement = new Element(AnchorAttribute.getBaseTag());
			attr.fillInto(attrElement);
		
			parent.appendChild(attrElement);

		}

		// Append contained xcfids.
		StringBuffer buffer = new StringBuffer();

		Element containedIdsElement = new Element("CONTAINEDIDS");
		int counter = 0;
		for (String id : xcfIds) {
			
			buffer.setLength(0);
			buffer.append("id");
			buffer.append(String.valueOf(counter));
			
			containedIdsElement.addAttribute(new Attribute(buffer.toString(), id));				

			counter++;

		}

		parent.appendChild(containedIdsElement);

		// Append anchored fields list.
		Element anchoredFieldsElement = new Element("ANCHOREDFIELDS");
		counter = 0;
		for (String field : anchoredFields) {
	
			buffer.setLength(0);
			buffer.append("field");
			buffer.append(String.valueOf(counter));
			
			anchoredFieldsElement.addAttribute(new Attribute(buffer.toString(), field));

			counter++;

		}

		parent.appendChild(anchoredFieldsElement);

		// Add ratings to document.
		Element ratingsElement = new Element("RATINGS");
		counter = 0;
		for (Map.Entry<String, Double> entry : ratings.entrySet()) {

			buffer.setLength(0);
			buffer.append("xcfid");
			buffer.append(String.valueOf(counter));
			
			ratingsElement.addAttribute(new Attribute(buffer.toString(), entry.getKey()));

			buffer.setLength(0);
			buffer.append("rating");
			buffer.append(String.valueOf(counter));

			ratingsElement.addAttribute(new Attribute(buffer.toString(), String.valueOf(entry.getValue())));

			counter++;

		}

		parent.appendChild(ratingsElement);

	}

	/** */
	public static void fromElement(Element element, CompositeAnchor cAnchor) {

		Type.fromElement(element, cAnchor);

		Attribute a = element.getAttribute("symbol");
		if (a != null)
			cAnchor.setSymbol(a.getValue());

		Nodes nodes = element.query("*");
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element elem = (Element) node;

				// Parsing of anchor attributes.			
				if (elem.getLocalName().equals(AnchorAttribute.getBaseTag())) {

					AnchorAttribute attr = new AnchorAttribute();
					AnchorAttribute.fromElement(elem, attr);

					cAnchor.addAttribute(attr);

				}	

				// Parse contained xcf ids.
				if (elem.getLocalName().equals("CONTAINEDIDS"))	 {

					int nrAttributes = elem.getAttributeCount();
					for (int j = 0; j < nrAttributes; j++) {
						cAnchor.addXcfId(elem.getAttribute(j).getValue());
					}

				}

				// Parse anchored fields.
				if (elem.getLocalName().equals("ANCHOREDFIELDS")) {
				
					int nrAttributes = elem.getAttributeCount();
					for (int j = 0; j < nrAttributes; j++) {
						cAnchor.addField(elem.getAttribute(j).getValue());
					}

				}		

				// Parse contained assignment values.
				if (elem.getLocalName().equals("RATINGS")) {

					StringBuffer buffer = new StringBuffer();

					int nrAttributes = elem.getAttributeCount() / 2;
					for (int j = 0; j < nrAttributes; j++) {
				
						buffer.setLength(0);
						buffer.append("xcfid");
						buffer.append(String.valueOf(j));
				
						String xcfId = elem.getAttributeValue(buffer.toString());
		
						buffer.setLength(0);
						buffer.append("rating");
						buffer.append(String.valueOf(j));		

						double rating = Double.parseDouble(elem.getAttributeValue(buffer.toString()));

						cAnchor.addAssignment(xcfId, rating);

					}

				}


			}

		}

	}		

}
