package de.unibi.airobots.btl.data;

import nu.xom.Element;
import nu.xom.ParsingException;

/**
 * Encapsulation of state changes from the global path planner.
 * 
 * @author jwienke
 */
public class GlobalPlannerStateChange extends Type {

	private static final String STATE_ELEMENT_NAME = "STATE";

	/**
	 * Possible state of a global path planner.
	 * 
	 * @author jwienke
	 */
	public static enum PlannerState {

		/**
		 * The planner now begins planning a new path.
		 */
		PLANNING,

		/**
		 * Planning finished successfully and the robots is now trying to move
		 * along the planned path. In this state the planner can switch back to
		 * <code>PLANNING</code> every time it detects a new obstacle on the
		 * planned path.
		 */
		FOLLOWING_PLAN,

		/**
		 * The planner could not find a path to the desired location. This state
		 * is only left if a new goal is specified.
		 */
		PATH_BLOCKED

	}

	private PlannerState newState;

	public GlobalPlannerStateChange() {
	}

	/**
	 * Returns the new state of the global planner this change indicates.
	 * 
	 * @return global planner state
	 */
	public PlannerState getNewState() {
		return newState;
	}

	/**
	 * Sets the new state of the global planner.
	 * 
	 * @param newState
	 *            new state of the planner
	 */
	public void setNewState(PlannerState newState) {
		this.newState = newState;
	}
	
	public static String getBaseTag() {
		return "GLOBALPLANNERSTATECHANGE";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {

		if (!(obj instanceof GlobalPlannerStateChange)) {
			return false;
		}

		GlobalPlannerStateChange other = (GlobalPlannerStateChange) obj;

		return super.equals(obj) && (other.newState.equals(newState));

	}

	public static void fromElement(Element element,
			GlobalPlannerStateChange type) throws ParsingException {

		Type.fromElement(element, type);

		Element stateElement = element.getFirstChildElement(STATE_ELEMENT_NAME);
		if (stateElement == null) {
			throw new ParsingException("Unable to find element with name "
					+ STATE_ELEMENT_NAME + ".");
		}

		String stateName = stateElement.getValue();
		if ((stateName == null) || stateName.trim().isEmpty()) {
			throw new ParsingException("No state found in element with name "
					+ STATE_ELEMENT_NAME + ".");
		}

		try {
			PlannerState newState = PlannerState.valueOf(stateName);
			type.setNewState(newState);
		} catch (IllegalArgumentException e) {
			throw new ParsingException("Unable to extract valid PlannerState.");
		} catch (NullPointerException e) {
			throw new ParsingException("Unable to extract valid PlannerState.");
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);
		
		Element stateElement = new Element(STATE_ELEMENT_NAME);
		parent.appendChild(stateElement);
		stateElement.appendChild(newState.name());
		
	}

}
