package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.common.ElementParser;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Base class for all sensor results containing (robot/world) positions (X, Y, theta).
 */
public class PositionData extends Type {

	private double X;
	private double Y;
	private double Yaw;

	/**
	 * Creates a position data object initialized with x, y and theta set to 0.0 and the timestamp
	 * set to the current time.
	 */
	public PositionData() {
		this(0.0, 0.0, 0.0, System.currentTimeMillis());
	}

	/**
	 * Creates a new position data object initialized with the given values.
	 * 
	 * @param x
	 *            The X coordinate [m]
	 * @param y
	 *            The Y coordinate [m]
	 * @param yaw
	 *            The yaw/rotation [rad]
	 * @param timestamp
	 *            The timestamp of this position object [ms since 1970]
	 */
	public PositionData(double x, double y, double yaw, Timestamp timestamp) {
		this.X = x;
		this.Y = y;
		this.Yaw = yaw;
		setTimestamp(timestamp);
	}

	public PositionData(double x, double y, double yaw, long timestamp) {
		this.X = x;
		this.Y = y;
		this.Yaw = yaw;
		setTimestamp(timestamp);
	}

	/**
	 * Sets the current x Position.
	 */
	public void setX(double X) {
		this.X = X;
	}

	/**
	 * Sets the current y Position.
	 */
	public void setY(double Y) {
		this.Y = Y;
	}

	/**
	 * Sets the current yaw.
	 */
	public void setYaw(double Yaw) {
		this.Yaw = Yaw;
	}

	/**
	 * Returns the current x Position of the robot in world coordinates.
	 * 
	 * @return The x position as double.
	 */
	public double getX() {
		return X;
	}

	/**
	 * Returns the current y Position of the robot in world coordinates
	 * 
	 * @return The y position as double.
	 */
	public double getY() {
		return Y;
	}

	public double getDistance(PositionData otherPosition) {
		double dx = X - otherPosition.getX();
		double dy = Y - otherPosition.getY();
		return Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));
	}

	public double getGlobalAngle(PositionData otherPosition) {
		double dx = otherPosition.getX() - X;
		double dy = otherPosition.getY() - Y;
		return Math.atan2(dy, dx);
	}

	public double getRelativeAngle(PositionData otherPosition) {
		double angle = getGlobalAngle(otherPosition) - Yaw;
		if (angle > Math.PI) {
			angle -= 2 * Math.PI;
		}
		if (angle < -Math.PI) {
			angle += 2 * Math.PI;
		}
		return angle;
	}

	/**
	 * Returns the current yaw [rad] of the robot in world coordinates.
	 * 
	 * Ranges from -PI to PI. 0.0 is returned of the robot is directed to the right. Positive values
	 * are returned if the robot is directed up and negative values if the robot is directed down.
	 * 
	 * @return The yaw as a double.
	 */
	public double getYaw() {
		return Yaw;
	}

	public static String getBaseTag() {
		return "ROBOTPOSITION";
	}

	public static void fromElement(Element element, PositionData type) throws ParsingException {

		Type.fromElement(element, type);

		Nodes positionNodes = element.query("POSITION");
		if (positionNodes.size() < 1 && !(positionNodes.get(0) instanceof Element)) {
			throw new ParsingException("Unable to find element with name POSITION.");
		}
		Element positionElement = (Element) positionNodes.get(0);

		type.setX(ElementParser.getDoubleAttributeValue(positionElement, "x"));
		type.setY(ElementParser.getDoubleAttributeValue(positionElement, "y"));
		type.setYaw(ElementParser.getDoubleAttributeValue(positionElement, "theta"));

	}

	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element positionElement = new Element("POSITION");
		parent.appendChild(positionElement);

		positionElement.addAttribute(new Attribute("x", String.valueOf(getX())));
		positionElement.addAttribute(new Attribute("y", String.valueOf(getY())));
		positionElement.addAttribute(new Attribute("theta", String.valueOf(getYaw())));

		positionElement.addAttribute(new Attribute("ref", "world"));
		positionElement.addAttribute(new Attribute("kind", "absolute"));

	}

	@Override
	public boolean equals(Object obj) {

		if (!(obj instanceof PositionData)) {
			return false;
		}

		PositionData other = (PositionData) obj;

		return super.equals(obj) && close(other.getX(), getX()) && close(other.getY(), getY())
				&& close(other.getYaw(), getYaw());

	}

	protected boolean close(double d1, double d2) {
		return Math.abs(d1 - d2) < 0.0001;
	}

}
